-- Create My_AutoBL Auto-Blacklist Accessory table
CREATE TABLE [dbo].[My_AutoBL](
	[ID] [int] IDENTITY(1,1) NOT NULL,
	[EmailFrom] [nvarchar](100) NOT NULL,
	[EmailTo] [nvarchar](100) NOT NULL,
 CONSTRAINT [PK_My_AutoBL] PRIMARY KEY CLUSTERED 
(
	[ID] ASC
)WITH (IGNORE_DUP_KEY = OFF) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX [IX_My_AutoBL_EmailFrom] ON [dbo].[My_AutoBL] 
(
	[EmailFrom] ASC
)WITH (IGNORE_DUP_KEY = OFF) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX [IX_My_AutoBL_EmailTo] ON [dbo].[My_AutoBL] 
(
	[EmailTo] ASC
)WITH (IGNORE_DUP_KEY = OFF) ON [PRIMARY]
GO

-- Create the stored procedure that handles the addition of entries to the 
-- My_AutoBL automatic blacklist and to perform routine cleanup of the quarantine database
-- replacing the one being performed by SpamFilter
CREATE PROCEDURE [dbo].[PurgeQuarantineAndAutoBL] 
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	-- SET NOCOUNT ON;

-- Locate and add to the My_AutoBL table all from/to email addresses that have been marked
-- by users to be deleted
	INSERT INTO My_AutoBL (EmailFrom, EmailTo)
		SELECT     tblQuarantine.EmailFrom, tblQuarantine.EmailTo
		FROM         My_AutoBL RIGHT OUTER JOIN tblQuarantine ON 
						My_AutoBL.EmailFrom = tblQuarantine.EmailFrom AND 
						My_AutoBL.EmailTo = tblQuarantine.EmailTo
		WHERE     (tblQuarantine.Expire = 1) AND (My_AutoBL.ID IS NULL)

-- Now delete all entries in the tblQuarantine that match the to/from email addresses that have been
-- added in the past to the My_AutoBL blacklist.
	DELETE FROM tblQuarantine
	FROM   My_AutoBL INNER JOIN	tblQuarantine ON 
			My_AutoBL.EmailFrom = tblQuarantine.EmailFrom AND
			My_AutoBL.EmailTo = tblQuarantine.EmailTo


-- Perform routine cleanup for the tblQuarantine by removing all emails older than the specified number
-- of days below (7 by default in the:
-- DATEDIFF(day, MsgDate, GETDATE()) > 7 
-- we are deleting 500 entries at a time to avoid locking rown in the database for too long
	SET ROWCOUNT 500
	delete_more1:
		UPDATE tblQuarantine SET Expire = 1 WHERE (Expire <>1) AND (DATEDIFF(day, MsgDate, GETDATE()) > 7)
	IF @@ROWCOUNT > 0 GOTO delete_more1
	SET ROWCOUNT 0

	SET ROWCOUNT 500
	delete_more2:
		DELETE FROM tblQuarantine WHERE tblQuarantine.Expire <> 0
	IF @@ROWCOUNT > 0 GOTO delete_more2
	SET ROWCOUNT 0

	SET ROWCOUNT 500
	delete_more3:
	   DELETE tblMsgs FROM tblMsgs LEFT JOIN tblQuarantine 
	ON tblMsgs.MsgID = tblQuarantine.MsgID WHERE (tblQuarantine.MsgID IS NULL)
	IF @@ROWCOUNT > 0 GOTO delete_more3
	SET ROWCOUNT 0
END

